import Flutter
import UIKit
import MRZScannerSDK

public class SwiftMrzflutterpluginPlugin: NSObject, FlutterPlugin, MRZScannerDelegate {
    
    var _mrzScannerController : MRZScannerController?
    var pendingResult: FlutterResult?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "mrzflutterplugin", binaryMessenger: registrar.messenger())
        let instance = SwiftMrzflutterpluginPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "startScanner":
            pendingResult = result
            startScanner()
            break
        case "resumeScanner":
            if _mrzScannerController != nil {
                _mrzScannerController?.resumeScanner()
            }
            break
        case "closeScanner":
            if _mrzScannerController != nil {
                _mrzScannerController?.closeScanner()
            }
            break
        case "setNightModeActive":
            if _mrzScannerController != nil && call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    _mrzScannerController?.setNightModeActive(boolVal)
                }
            }
            break
        case "getSdkVersion":
            result(MRZScannerController.getSDKVersion())
            break
        case "registerWithLicenceKey":
            if call.arguments != nil {
                if let str = call.arguments as? String {
                    MRZScannerController.registerLicense(withKey: str)
                }
            }
            break
        case "scanFromGallery":
            pendingResult = result
            scanFromGallery()
            break
        case "setDateFormat":
            if call.arguments != nil {
                if let str = call.arguments as? String {
                    MRZScannerController.setDateFormat(str)
                }
            }
            break
        case "setPassportActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setPassportActive(boolVal)
                }
            }
            break
        case "setIDActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setIDActive(boolVal)
                }
            }
            break
        case "setVisaActive":
            if call.arguments != nil {
                if let boolVal = call.arguments as? Bool {
                    MRZScannerController.setVisaActive(boolVal)
                }
            }
            break
        default:
            result(FlutterMethodNotImplemented)
        }
    }
    
    func startScanner(){
        let currentVC = UIApplication.shared.keyWindow?.rootViewController
        _mrzScannerController = MRZScannerController()
        _mrzScannerController!.delegate = self
        currentVC?.addChild(_mrzScannerController!)
        _mrzScannerController!.initUI(currentVC)
    }
    
    func scanFromGallery(){
        let currentVC = UIApplication.shared.keyWindow?.rootViewController
        MRZScannerController.scan(fromGallery: currentVC, delegate: self)
    }
    
    public func successfulScan(withResult result: MRZResultDataModel!) {
        if pendingResult != nil {
            let jsonResult = result.toJSON()
            
            if result.portrait != nil || result.signature != nil || result.fullImage != nil {
                let data = jsonResult?.data(using: .utf8)

                do {
                    var jsonDictionary = try JSONSerialization.jsonObject(with: data!, options:[]) as? Dictionary<String, Any?>
                    
                    if result.portrait != nil {
                        let imgData = result.portrait.pngData()
                        let base64Image = imgData?.base64EncodedData()
                        jsonDictionary?["portrait"] = String.init(data: base64Image!, encoding: .utf8)
                    }
                    
                    if result.signature != nil {
                        let imgData = result.signature.pngData()
                        let base64Image = imgData?.base64EncodedData()
                        jsonDictionary?["signature"] = String.init(data: base64Image!, encoding: .utf8)
                    }
                    
                    if result.fullImage != nil {
                        let imgData = result.fullImage.pngData()
                        let base64Image = imgData?.base64EncodedData()
                        jsonDictionary?["full_image"] = String.init(data: base64Image!, encoding: .utf8)
                    }
                    
                    if let theJSONData = try? JSONSerialization.data(withJSONObject: jsonDictionary!,
                                                                            options: [])
                    {
                        let theJSONText = String(data: theJSONData, encoding: .utf8)
                        pendingResult!(theJSONText)
                    } else {
                        pendingResult!(jsonResult)
                    }
                } catch {
                    pendingResult!(jsonResult)
                }
            } else {
                pendingResult!(jsonResult)
            }
            
            
            _mrzScannerController?.dismiss(animated: true, completion: nil)
        }
    }
    
    public func scannerWasDismissed() {
        if pendingResult != nil {
            pendingResult!(FlutterError.init(code: "scannerWasDismissed", message: "scannerWasDismissed", details: nil))
        }
    }
    
    public func permissionsWereDenied() {
        if pendingResult != nil {
            pendingResult!(FlutterError.init(code: "permissionsWereDenied", message: "permissionsWereDenied", details: nil))
        }
    }
    
    public func scanImageFailed() {
        if pendingResult != nil {
            pendingResult!(FlutterError.init(code: "scanImageFailed", message: "scanImageFailed", details: nil))
        }
    }
    
}
